@@ -1,6 +1,6 @@
 /* gpgv.c - The GnuPG signature verify utility
- * Copyright (C) 1998, 1999, 2000, 2001, 2002,
- *               2005 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2005,
+ *               2006 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -270,16 +270,35 @@ get_ownertrust (PKT_public_key *pk)
 }
 
 
-/* Stub:
+/* Stubs:
  * Because we only work with trusted keys, it does not make sense to
  * get them from a keyserver
  */
+
+struct keyserver_spec *
+keyserver_match(struct keyserver_spec *spec) { return NULL; }
+
 int
 keyserver_import_keyid( u32 *keyid, void *dummy )
 {
     return -1;
 }
 
+int
+keyserver_import_cert(const char *name) { return -1; }
+
+int
+keyserver_import_pka(const char *name,unsigned char *fpr) { return -1; }
+
+int
+keyserver_import_name(const char *name,struct keyserver_spec *spec)
+{
+  return -1;
+}
+
+int
+keyserver_import_ldap(const char *name) { return -1; }
+
 /* Stub:
  * No encryption here but mainproc links to these functions.
  */
@@ -333,6 +352,13 @@ passphrase_to_dek( u32 *keyid, int pubkey_algo,
 }
 
 struct keyserver_spec *parse_preferred_keyserver(PKT_signature *sig) {return NULL;}
+struct keyserver_spec *parse_keyserver_uri(const char *uri,int require_scheme,
+                                           const char *configname,
+                                           unsigned int configlineno)
+{
+  return NULL;
+}
+
 void free_keyserver_spec(struct keyserver_spec *keyserver) {}
 
 /* Stubs to avoid linking to photoid.c */
