@@ -55,29 +55,31 @@ struct kidlist_item {
  * Structure to hold the context
  */
 typedef struct mainproc_context *CTX;
-struct mainproc_context {
-    struct mainproc_context *anchor;  /* may be useful in the future */
-    PKT_public_key *last_pubkey;
-    PKT_secret_key *last_seckey;
-    PKT_user_id     *last_user_id;
-    md_filter_context_t mfx;
-    int sigs_only;   /* process only signatures and reject all other stuff */
-    int encrypt_only; /* process only encrytion messages */
-    STRLIST signed_data;
-    const char *sigfilename;
-    DEK *dek;
-    int last_was_session_key;
-    KBNODE list;   /* the current list of packets */
-    int have_data;
-    IOBUF iobuf;    /* used to get the filename etc. */
-    int trustletter; /* temp usage in list_node */
-    ulong symkeys;
-    struct kidlist_item *pkenc_list;	/* list of encryption packets */
-    struct {
-        int op;
-        int stop_now;
-    } pipemode;
-    int any_sig_seen;  /* Set to true if a signature packet has been seen. */
+struct mainproc_context
+{
+  struct mainproc_context *anchor;  /* May be useful in the future. */
+  PKT_public_key *last_pubkey;
+  PKT_secret_key *last_seckey;
+  PKT_user_id     *last_user_id;
+  md_filter_context_t mfx;
+  int sigs_only;    /* Process only signatures and reject all other stuff. */
+  int encrypt_only; /* Process only encryption messages. */
+  STRLIST signed_data;
+  const char *sigfilename;
+  DEK *dek;
+  int last_was_session_key;
+  KBNODE list;      /* The current list of packets. */
+  int have_data;
+  IOBUF iobuf;      /* Used to get the filename etc. */
+  int trustletter;  /* Temporary usage in list_node. */
+  ulong symkeys;
+  struct kidlist_item *pkenc_list; /* List of encryption packets. */
+  struct 
+  {
+    int op;
+    int stop_now;
+  } pipemode;
+  int any_sig_seen;  /* Set to true if a signature packet has been seen. */
 };
 
 
@@ -96,7 +98,7 @@ release_list( CTX c )
     release_kbnode( c->list );
     while( c->pkenc_list ) {
 	struct kidlist_item *tmp = c->pkenc_list->next;
-	m_free( c->pkenc_list );
+	xfree( c->pkenc_list );
 	c->pkenc_list = tmp;
     }
     c->pkenc_list = NULL;
@@ -105,7 +107,7 @@ release_list( CTX c )
     c->last_was_session_key = 0;
     c->pipemode.op = 0;
     c->pipemode.stop_now = 0;
-    m_free(c->dek); c->dek = NULL;
+    xfree(c->dek); c->dek = NULL;
 }
 
 
@@ -307,16 +309,31 @@ proc_symkey_enc( CTX c, PACKET *pkt )
 
 	if(opt.override_session_key)
 	  {
-	    c->dek = m_alloc_clear( sizeof *c->dek );
+	    c->dek = xmalloc_clear( sizeof *c->dek );
 	    if(get_override_session_key(c->dek, opt.override_session_key))
 	      {
-		m_free(c->dek);
+		xfree(c->dek);
 		c->dek = NULL;
 	      }
 	  }
 	else
 	  {
-	    c->dek=passphrase_to_dek(NULL, 0, algo, &enc->s2k, 0, NULL, NULL);
+            int canceled;
+
+	    c->dek = passphrase_to_dek (NULL, 0, algo, &enc->s2k, 0,
+                                        NULL, &canceled);
+            if (canceled)
+              {
+                /* For unknown reasons passphrase_to_dek does only
+                   return NULL if a new passphrase has been requested
+                   and has not been repeated correctly.  Thus even
+                   with a cancel requested (by means of the gpg-agent)
+                   it won't return NULL but an empty passphrase.  We
+                   take the most conservative approach for now and
+                   work around it right here. */
+                xfree (c->dek);
+                c->dek = NULL;
+              }
 
 	    if(c->dek)
 	      {
@@ -334,7 +351,7 @@ proc_symkey_enc( CTX c, PACKET *pkt )
 		    if(symkey_decrypt_seskey(c->dek, enc->seskey,
 					     enc->seskeylen))
 		      {
-			m_free(c->dek);
+			xfree(c->dek);
 			c->dek=NULL;
 		      }
 		  }
@@ -374,26 +391,28 @@ proc_pubkey_enc( CTX c, PACKET *pkt )
     if( !opt.list_only && opt.override_session_key ) {
 	/* It does not make much sense to store the session key in
 	 * secure memory because it has already been passed on the
-	 * command line and the GCHQ knows about it */
-	c->dek = m_alloc_clear( sizeof *c->dek );
+	 * command line and the GCHQ knows about it.  */
+	c->dek = xmalloc_clear( sizeof *c->dek );
 	result = get_override_session_key ( c->dek, opt.override_session_key );
 	if ( result ) {
-	    m_free(c->dek); c->dek = NULL;
+	    xfree(c->dek); c->dek = NULL;
 	}
     }
     else if( is_ELGAMAL(enc->pubkey_algo)
 	|| enc->pubkey_algo == PUBKEY_ALGO_DSA
 	|| is_RSA(enc->pubkey_algo)  ) {
+      /* FIXME:  strore this all in a list and process it later */
+
 	if ( !c->dek && ((!enc->keyid[0] && !enc->keyid[1])
                           || opt.try_all_secrets
 			  || !seckey_available( enc->keyid )) ) {
 	    if( opt.list_only )
 		result = -1;
 	    else {
-		c->dek = m_alloc_secure_clear( sizeof *c->dek );
+		c->dek = xmalloc_secure_clear( sizeof *c->dek );
 		if( (result = get_session_key( enc, c->dek )) ) {
 		    /* error: delete the DEK */
-		    m_free(c->dek); c->dek = NULL;
+		    xfree(c->dek); c->dek = NULL;
 		}
 	    }
 	}
@@ -408,7 +427,7 @@ proc_pubkey_enc( CTX c, PACKET *pkt )
     else
       {
         /* store it for later display */
-	struct kidlist_item *x = m_alloc( sizeof *x );
+	struct kidlist_item *x = xmalloc( sizeof *x );
 	x->kid[0] = enc->keyid[0];
 	x->kid[1] = enc->keyid[1];
 	x->pubkey_algo = enc->pubkey_algo;
@@ -442,7 +461,7 @@ print_pkenc_list( struct kidlist_item *list, int failed )
             continue;
 
         algstr = pubkey_algo_to_string( list->pubkey_algo );
-        pk = m_alloc_clear( sizeof *pk );
+        pk = xmalloc_clear( sizeof *pk );
 
 	if( !algstr )
 	    algstr = "[?]";
@@ -455,7 +474,7 @@ print_pkenc_list( struct kidlist_item *list, int failed )
 		      strtimestamp(pk->timestamp) );
 	    p=get_user_id_native(list->kid);
 	    fprintf(log_stream(),_("      \"%s\"\n"),p);
-	    m_free(p);
+	    xfree(p);
 	  }
 	else
 	  log_info(_("encrypted with %s key, ID %s\n"),
@@ -493,6 +512,9 @@ proc_encrypted( CTX c, PACKET *pkt )
         print_pkenc_list ( c->pkenc_list, 0 );
       }
 
+    /* FIXME: Figure out the session key by looking at all pkenc packets. */
+
+
     write_status( STATUS_BEGIN_DECRYPTION );
 
     /*log_debug("dat: %sencrypted data\n", c->dek?"":"conventional ");*/
@@ -504,11 +526,11 @@ proc_encrypted( CTX c, PACKET *pkt )
 
 	if(opt.override_session_key)
 	  {
-	    c->dek = m_alloc_clear( sizeof *c->dek );
+	    c->dek = xmalloc_clear( sizeof *c->dek );
 	    result=get_override_session_key(c->dek, opt.override_session_key);
 	    if(result)
 	      {
-		m_free(c->dek);
+		xfree(c->dek);
 		c->dek = NULL;
 	      }
 	  }
@@ -564,7 +586,7 @@ proc_encrypted( CTX c, PACKET *pkt )
 	if(opt.show_session_key)
 	  {
 	    int i;
-	    char *buf = m_alloc ( c->dek->keylen*2 + 20 );
+	    char *buf = xmalloc ( c->dek->keylen*2 + 20 );
 	    sprintf ( buf, "%d:", c->dek->algo );
 	    for(i=0; i < c->dek->keylen; i++ )
 	      sprintf(buf+strlen(buf), "%02X", c->dek->key[i] );
@@ -583,7 +605,7 @@ proc_encrypted( CTX c, PACKET *pkt )
 	/* Hmmm: does this work when we have encrypted using multiple
 	 * ways to specify the session key (symmmetric and PK)*/
     }
-    m_free(c->dek); c->dek = NULL;
+    xfree(c->dek); c->dek = NULL;
     free_packet(pkt);
     c->last_was_session_key = 0;
     write_status( STATUS_END_DECRYPTION );
@@ -645,7 +667,8 @@ proc_plaintext( CTX c, PACKET *pkt )
             for( data++, datalen--; datalen; datalen--, data++ )
 	      md_enable( c->mfx.md, *data );
             any = 1;
-            break;  /* no pass signature packets are expected */
+            break;  /* Stop here as one-pass signature packets are not
+                       expected.  */
 	  }
 	else if(n->pkt->pkttype==PKT_SIGNATURE)
 	  {
@@ -725,7 +748,9 @@ proc_compressed( CTX c, PACKET *pkt )
     int rc;
 
     /*printf("zip: compressed data packet\n");*/
-    if( c->sigs_only )
+    if( !zd->algorithm )
+      rc=G10ERR_COMPR_ALGO;
+    else if( c->sigs_only )
 	rc = handle_compressed( c, zd, proc_compressed_cb, c );
     else if( c->encrypt_only )
 	rc = handle_compressed( c, zd, proc_encrypt_cb, c );
@@ -912,10 +937,12 @@ list_node( CTX c, KBNODE node )
 		    putchar('\n');
 		    if( opt.fingerprint && !any )
 			print_fingerprint( pk, NULL, 0 );
-		    if( node->next
+		    if( opt.with_colons
+                        && node->next
 			&& node->next->pkt->pkttype == PKT_RING_TRUST ) {
 			printf("rtv:2:%u:\n",
-				 node->next->pkt->pkt.ring_trust->trustval );
+                               node->next->pkt->pkt.ring_trust?
+                               node->next->pkt->pkt.ring_trust->trustval : 0);
 		    }
 		    any=1;
 		}
@@ -1088,7 +1115,7 @@ list_node( CTX c, KBNODE node )
 	else if( !opt.fast_list_mode ) {
 	    p = get_user_id( sig->keyid, &n );
 	    print_string( stdout, p, n, opt.with_colons );
-	    m_free(p);
+	    xfree(p);
 	}
 	if( opt.with_colons )
 	    printf(":%02x%c:", sig->sig_class, sig->flags.exportable?'x':'l');
@@ -1104,11 +1131,11 @@ int
 proc_packets( void *anchor, IOBUF a )
 {
     int rc;
-    CTX c = m_alloc_clear( sizeof *c );
+    CTX c = xmalloc_clear( sizeof *c );
 
     c->anchor = anchor;
     rc = do_proc_packets( c, a );
-    m_free( c );
+    xfree( c );
     return rc;
 }
 
@@ -1118,7 +1145,7 @@ int
 proc_signature_packets( void *anchor, IOBUF a,
 			STRLIST signedfiles, const char *sigfilename )
 {
-    CTX c = m_alloc_clear( sizeof *c );
+    CTX c = xmalloc_clear( sizeof *c );
     int rc;
 
     c->anchor = anchor;
@@ -1126,15 +1153,15 @@ proc_signature_packets( void *anchor, IOBUF a,
     c->signed_data = signedfiles;
     c->sigfilename = sigfilename;
     rc = do_proc_packets( c, a );
- 
+
     /* If we have not encountered any signature we print an error
        messages, send a NODATA status back and return an error code.
-       Using log_error is required becuase verify_files does not check
+       Using log_error is required because verify_files does not check
        error codes for each file but we want to terminate the process
        with an error. */ 
     if (!rc && !c->any_sig_seen)
       {
- 	write_status_text (STATUS_NODATA, "4");
+	write_status_text (STATUS_NODATA, "4");
         log_error (_("no signature found\n"));
         rc = G10ERR_NO_DATA;
       }
@@ -1145,20 +1172,20 @@ proc_signature_packets( void *anchor, IOBUF a,
     if (!rc && c->anchor && c->any_sig_seen)
       c->anchor->any_sig_seen = 1;
 
-    m_free( c );
+    xfree( c );
     return rc;
 }
 
 int
 proc_encryption_packets( void *anchor, IOBUF a )
 {
-    CTX c = m_alloc_clear( sizeof *c );
+    CTX c = xmalloc_clear( sizeof *c );
     int rc;
 
     c->anchor = anchor;
     c->encrypt_only = 1;
     rc = do_proc_packets( c, a );
-    m_free( c );
+    xfree( c );
     return rc;
 }
 
@@ -1166,7 +1193,7 @@ proc_encryption_packets( void *anchor, IOBUF a )
 int
 do_proc_packets( CTX c, IOBUF a )
 {
-    PACKET *pkt = m_alloc( sizeof *pkt );
+    PACKET *pkt = xmalloc( sizeof *pkt );
     int rc=0;
     int any_data=0;
     int newpkt;
@@ -1274,7 +1301,7 @@ do_proc_packets( CTX c, IOBUF a )
 	if( newpkt == -1 )
 	    ;
 	else if( newpkt ) {
-	    pkt = m_alloc( sizeof *pkt );
+	    pkt = xmalloc( sizeof *pkt );
 	    init_packet(pkt);
 	}
 	else
@@ -1296,14 +1323,79 @@ do_proc_packets( CTX c, IOBUF a )
 
   leave:
     release_list( c );
-    m_free(c->dek);
+    xfree(c->dek);
     free_packet( pkt );
-    m_free( pkt );
+    xfree( pkt );
     free_md_filter_context( &c->mfx );
     return rc;
 }
 
 
+/* Helper for pka_uri_from_sig to parse the to-be-verified address out
+   of the notation data. */
+static pka_info_t *
+get_pka_address (PKT_signature *sig)
+{
+  pka_info_t *pka = NULL;
+  struct notation *nd,*notation;
+
+  notation=sig_to_notation(sig);
+
+  for(nd=notation;nd;nd=nd->next)
+    {
+      if(strcmp(nd->name,"pka-address@gnupg.org")!=0)
+        continue; /* Not the notation we want. */
+
+      /* For now we only use the first valid PKA notation. In future
+	 we might want to keep additional PKA notations in a linked
+	 list. */
+      if (is_valid_mailbox (nd->value))
+	{
+	  pka = xmalloc (sizeof *pka + strlen(nd->value));
+	  pka->valid = 0;
+	  pka->checked = 0;
+	  pka->uri = NULL;
+	  strcpy (pka->email, nd->value);
+	  break;
+	}
+    }
+
+  free_notation(notation);
+
+  return pka;
+}
+
+
+/* Return the URI from a DNS PKA record.  If this record has already
+   be retrieved for the signature we merely return it; if not we go
+   out and try to get that DNS record. */
+static const char *
+pka_uri_from_sig (PKT_signature *sig)
+{
+  if (!sig->flags.pka_tried)
+    {
+      assert (!sig->pka_info);
+      sig->flags.pka_tried = 1;
+      sig->pka_info = get_pka_address (sig);
+      if (sig->pka_info)
+        {
+          char *uri;
+
+          uri = get_pka_info (sig->pka_info->email, sig->pka_info->fpr);
+          if (uri)
+            {
+              sig->pka_info->valid = 1;
+              if (!*uri)
+                xfree (uri);
+              else
+                sig->pka_info->uri = uri;
+            }
+        }
+    }
+  return sig->pka_info? sig->pka_info->uri : NULL;
+}
+
+
 static int
 check_sig_and_print( CTX c, KBNODE node )
 {
@@ -1477,11 +1569,39 @@ check_sig_and_print( CTX c, KBNODE node )
 	  }
       }
 
-    /* If the preferred keyserver thing above didn't work, this is a
-       second try. */
+    /* If the preferred keyserver thing above didn't work, our second
+       try is to use the URI from a DNS PKA record. */
+    if ( rc == G10ERR_NO_PUBKEY 
+	 && opt.keyserver_options.options&KEYSERVER_AUTO_KEY_RETRIEVE
+         && opt.keyserver_options.options&KEYSERVER_HONOR_PKA_RECORD)
+      {
+        const char *uri = pka_uri_from_sig (sig);
+        
+        if (uri)
+          {
+            /* FIXME: We might want to locate the key using the
+               fingerprint instead of the keyid. */
+            int res;
+            struct keyserver_spec *spec;
+            
+            spec = parse_keyserver_uri (uri, 1, NULL, 0);
+            if (spec)
+              {
+                glo_ctrl.in_auto_key_retrieve++;
+                res = keyserver_import_keyid (sig->keyid, spec);
+                glo_ctrl.in_auto_key_retrieve--;
+                free_keyserver_spec (spec);
+                if (!res)
+                  rc = do_check_sig(c, node, NULL, &is_expkey, &is_revkey );
+              }
+          }
+      }
+
+    /* If the preferred keyserver thing above didn't work and we got
+       no information from the DNS PKA, this is a third try. */
 
     if( rc == G10ERR_NO_PUBKEY && opt.keyserver
-	&& (opt.keyserver_options.options&KEYSERVER_AUTO_KEY_RETRIEVE))
+	&& opt.keyserver_options.options&KEYSERVER_AUTO_KEY_RETRIEVE)
       {
 	int res;
 
@@ -1560,7 +1680,7 @@ check_sig_and_print( CTX c, KBNODE node )
 	    else
 	      log_info(_("Good signature from \"%s\""),p);
 
-	    m_free(p);
+	    xfree(p);
 
 	    if(opt.verify_options&VERIFY_SHOW_UID_VALIDITY)
 	      fprintf(log_stream()," [%s]\n",trust_value_to_string(valid));
@@ -1599,7 +1719,7 @@ check_sig_and_print( CTX c, KBNODE node )
 	      p=utf8_to_native(un->pkt->pkt.user_id->name,
                                un->pkt->pkt.user_id->len,0);
 	    else
-	      p=m_strdup("[?]");
+	      p=xstrdup("[?]");
 
 	    if(rc)
 	      log_info(_("BAD signature from \"%s\""),p);
@@ -1643,7 +1763,7 @@ check_sig_and_print( CTX c, KBNODE node )
 		p=utf8_to_native(un->pkt->pkt.user_id->name,
 				 un->pkt->pkt.user_id->len,0);
 		log_info(_("                aka \"%s\""),p);
-		m_free(p);
+		xfree(p);
 
 		if(opt.verify_options&VERIFY_SHOW_UID_VALIDITY)
 		  {
@@ -1686,7 +1806,7 @@ check_sig_and_print( CTX c, KBNODE node )
 
 	if( !rc && is_status_enabled() ) {
 	    /* print a status response with the fingerprint */
-	    PKT_public_key *vpk = m_alloc_clear( sizeof *vpk );
+	    PKT_public_key *vpk = xmalloc_clear( sizeof *vpk );
 
 	    if( !get_pubkey( vpk, sig->keyid ) ) {
 		byte array[MAX_FINGERPRINT_LEN], *p;
@@ -1714,7 +1834,7 @@ check_sig_and_print( CTX c, KBNODE node )
                    akid[0] = vpk->main_keyid[0];
                    akid[1] = vpk->main_keyid[1];
                    free_public_key (vpk);
-                   vpk = m_alloc_clear( sizeof *vpk );
+                   vpk = xmalloc_clear( sizeof *vpk );
                    if (get_pubkey (vpk, akid)) {
                      /* impossible error, we simply return a zeroed out fpr */
                      n = MAX_FINGERPRINT_LEN < 20? MAX_FINGERPRINT_LEN : 20;
@@ -1731,8 +1851,12 @@ check_sig_and_print( CTX c, KBNODE node )
 	    free_public_key( vpk );
 	}
 
-	if( !rc )
+	if (!rc)
+          {
+	    if(opt.verify_options&VERIFY_PKA_LOOKUPS)
+	      pka_uri_from_sig (sig); /* Make sure PKA info is available. */
 	    rc = check_signatures_trust( sig );
+          }
 
 	if(sig->flags.expired)
 	  {
@@ -1813,7 +1937,8 @@ proc_tree( CTX c, KBNODE node )
 	    /* prepare to create all requested message digests */
 	    c->mfx.md = md_open(0, 0);
 
-	    /* fixme: why looking for the signature packet and not 1passpacket*/
+	    /* fixme: why looking for the signature packet and not the
+               one-pass packet? */
 	    for( n1 = node; (n1 = find_next_kbnode(n1, PKT_SIGNATURE )); ) {
 		md_enable( c->mfx.md, n1->pkt->pkt.signature->digest_algo);
 	    }
