@@ -126,7 +126,7 @@ size_t iobuf_temp_to_buffer( IOBUF a, byte *buffer, size_t buflen );
 void iobuf_unget_and_close_temp( IOBUF a, IOBUF temp );
 
 int  iobuf_get_fd (IOBUF a);
-off_t iobuf_get_filelength( IOBUF a );
+off_t iobuf_get_filelength (IOBUF a, int *overflow);
 #define IOBUF_FILELENGTH_LIMIT 0xffffffff
 const char *iobuf_get_real_fname( IOBUF a );
 const char *iobuf_get_fname( IOBUF a );
@@ -135,10 +135,10 @@ void iobuf_set_partial_block_mode( IOBUF a, size_t len );
 
 int iobuf_translate_file_handle ( int fd, int for_write );
 
-/* get a byte form the iobuf; must check for eof prior to this function
- * this function returns values in the range 0 .. 255 or -1 to indicate EOF
+/* Get a byte form the iobuf; must check for eof prior to this function.
+ * This function returns values in the range 0 .. 255 or -1 to indicate EOF
  * iobuf_get_noeof() does not return -1 to indicate EOF, but masks the
- * returned value to be in the range 0 ..255.
+ * returned value to be in the range 0..255.
  */
 #define iobuf_get(a)  \
      (	((a)->nofast || (a)->d.start >= (a)->d.len )?  \
