@@ -1,5 +1,5 @@
 /* ksutil.c - general keyserver utility functions
- * Copyright (C) 2004, 2005 Free Software Foundation, Inc.
+ * Copyright (C) 2004, 2005, 2006 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -29,10 +29,8 @@
 #ifdef HAVE_LIBCURL
 #include <curl/curl.h>
 #else
-#ifdef FAKE_CURL
 #include "curl-shim.h"
 #endif
-#endif
 #include "keyserver.h"
 #include "ksutil.h"
 
@@ -46,6 +44,7 @@ int register_timeout(void) {return 0;}
 static void
 catch_alarm(int foo)
 {
+  (void)foo;
   _exit(KEYSERVER_TIMEOUT);
 }
 
@@ -85,6 +84,8 @@ init_ks_options(void)
   if(opt)
     {
       opt->action=KS_UNKNOWN;
+      opt->flags.include_revoked=1;
+      opt->flags.include_subkeys=1;
       opt->flags.check_cert=1;
       opt->timeout=DEFAULT_KEYSERVER_TIMEOUT;
       opt->path=strdup("/");
@@ -139,6 +140,8 @@ parse_ks_options(char *line,struct ks_options *opt)
 
       if(strcasecmp(command,"get")==0)
 	opt->action=KS_GET;
+      else if(strcasecmp(command,"getname")==0)
+	opt->action=KS_GETNAME;
       else if(strcasecmp(command,"send")==0)
 	opt->action=KS_SEND;
       else if(strcasecmp(command,"search")==0)
@@ -310,6 +313,7 @@ ks_action_to_string(enum ks_action action)
     {
     case KS_UNKNOWN: return "UNKNOWN";
     case KS_GET:     return "GET";
+    case KS_GETNAME: return "GETNAME";
     case KS_SEND:    return "SEND";
     case KS_SEARCH:  return "SEARCH";
     }
@@ -335,18 +339,99 @@ print_nocr(FILE *stream,const char *str)
     }
 }
 
-#if defined (HAVE_LIBCURL) || defined (FAKE_CURL)
+enum ks_search_type
+classify_ks_search(const char **search)
+{
+  switch(**search)
+    {
+    case '*':
+      (*search)++;
+      return KS_SEARCH_SUBSTR;
+    case '=':
+      (*search)++;
+      return KS_SEARCH_EXACT;
+    case '<':
+      (*search)++;
+      return KS_SEARCH_MAIL;
+    case '@':
+      (*search)++;
+      return KS_SEARCH_MAILSUB;
+    case '0':
+      if((*search)[1]=='x')
+	{
+	  if(strlen(*search)==10
+	     && strspn(*search,"abcdefABCDEF1234567890x")==10)
+	    {
+	      (*search)+=2;
+	      return KS_SEARCH_KEYID_SHORT;
+	    }
+	  else if(strlen(*search)==18
+		  && strspn(*search,"abcdefABCDEF1234567890x")==18)
+	    {
+	      (*search)+=2;
+	      return KS_SEARCH_KEYID_LONG;
+	    }
+	}
+      /* fall through */
+    default:
+      return KS_SEARCH_SUBSTR;
+    }
+}
+
 int
 curl_err_to_gpg_err(CURLcode error)
 {
   switch(error)
     {
-    case CURLE_FTP_COULDNT_RETR_FILE: return KEYSERVER_KEY_NOT_FOUND;
+    case CURLE_OK:                    return KEYSERVER_OK;
     case CURLE_UNSUPPORTED_PROTOCOL:  return KEYSERVER_SCHEME_NOT_FOUND;
+    case CURLE_COULDNT_CONNECT:       return KEYSERVER_UNREACHABLE;
+    case CURLE_FTP_COULDNT_RETR_FILE: return KEYSERVER_KEY_NOT_FOUND;
     default: return KEYSERVER_INTERNAL_ERROR;
     }
 }
 
+#define B64 "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/"
+
+static void
+curl_armor_writer(const unsigned char *buf,size_t size,void *cw_ctx)
+{
+  struct curl_writer_ctx *ctx=cw_ctx;
+  size_t idx=0;
+
+  while(idx<size)
+    {
+      for(;ctx->armor_remaining<3 && idx<size;ctx->armor_remaining++,idx++)
+	ctx->armor_ctx[ctx->armor_remaining]=buf[idx];
+
+      if(ctx->armor_remaining==3)
+	{
+	  /* Top 6 bytes of ctx->armor_ctx[0] */
+	  fputc(B64[(ctx->armor_ctx[0]>>2)&0x3F],ctx->stream);
+	  /* Bottom 2 bytes of ctx->armor_ctx[0] and top 4 bytes of
+	     ctx->armor_ctx[1] */
+	  fputc(B64[(((ctx->armor_ctx[0]<<4)&0x30)
+		     |((ctx->armor_ctx[1]>>4)&0x0F))&0x3F],ctx->stream);
+	  /* Bottom 4 bytes of ctx->armor_ctx[1] and top 2 bytes of
+	     ctx->armor_ctx[2] */
+	  fputc(B64[(((ctx->armor_ctx[1]<<2)&0x3C)
+		     |((ctx->armor_ctx[2]>>6)&0x03))&0x3F],ctx->stream);
+	  /* Bottom 6 bytes of ctx->armor_ctx[2] */
+	  fputc(B64[(ctx->armor_ctx[2]&0x3F)],ctx->stream);
+
+	  ctx->linelen+=4;
+	  if(ctx->linelen>=70)
+	    {
+	      fputc('\n',ctx->stream);
+	      ctx->linelen=0;
+	    }
+
+	  ctx->armor_remaining=0;
+	}
+    }
+
+}
+
 size_t
 curl_writer(const void *ptr,size_t size,size_t nmemb,void *cw_ctx)
 {
@@ -354,52 +439,102 @@ curl_writer(const void *ptr,size_t size,size_t nmemb,void *cw_ctx)
   const char *buf=ptr;
   size_t i;
 
-  if(!ctx->initialized)
+  if(!ctx->flags.initialized)
     {
-      ctx->marker=BEGIN;
-      ctx->initialized=1;
-    }
+      if(size*nmemb==0)
+	return 0;
 
-  /* scan the incoming data for our marker */
-  for(i=0;!ctx->done && i<(size*nmemb);i++)
-    {
-      if(buf[i]==ctx->marker[ctx->markeridx])
+      /* The object we're fetching is in binary form */
+      if(*buf&0x80)
 	{
-	  ctx->markeridx++;
-	  if(ctx->marker[ctx->markeridx]=='\0')
-	    {
-	      if(ctx->begun)
-		ctx->done=1;
-	      else
-		{
-		  /* We've found the BEGIN marker, so now we're looking
-		     for the END marker. */
-		  ctx->begun=1;
-		  ctx->marker=END;
-		  ctx->markeridx=0;
-		  fprintf(ctx->stream,BEGIN);
-		  continue;
-		}
-	    }
+	  ctx->flags.armor=1;
+	  fprintf(ctx->stream,BEGIN"\n\n");
 	}
       else
-	ctx->markeridx=0;
+	ctx->marker=BEGIN;
 
-      if(ctx->begun)
+      ctx->flags.initialized=1;
+    }
+
+  if(ctx->flags.armor)
+    curl_armor_writer(ptr,size*nmemb,cw_ctx);
+  else
+    {
+      /* scan the incoming data for our marker */
+      for(i=0;!ctx->flags.done && i<(size*nmemb);i++)
 	{
-	  /* Canonicalize CRLF to just LF by stripping CRs.  This
-	     actually makes sense, since on Unix-like machines LF is
-	     correct, and on win32-like machines, our output buffer is
-	     opened in textmode and will re-canonicalize line endings
-	     back to CRLF.  Since we only need to handle armored keys,
-	     we don't have to worry about odd cases like CRCRCR and
-	     the like. */
+	  if(buf[i]==ctx->marker[ctx->markeridx])
+	    {
+	      ctx->markeridx++;
+	      if(ctx->marker[ctx->markeridx]=='\0')
+		{
+		  if(ctx->flags.begun)
+		    ctx->flags.done=1;
+		  else
+		    {
+		      /* We've found the BEGIN marker, so now we're
+			 looking for the END marker. */
+		      ctx->flags.begun=1;
+		      ctx->marker=END;
+		      ctx->markeridx=0;
+		      fprintf(ctx->stream,BEGIN);
+		      continue;
+		    }
+		}
+	    }
+	  else
+	    ctx->markeridx=0;
 
-	  if(buf[i]!='\r')
-	    fputc(buf[i],ctx->stream);
+	  if(ctx->flags.begun)
+	    {
+	      /* Canonicalize CRLF to just LF by stripping CRs.  This
+		 actually makes sense, since on Unix-like machines LF
+		 is correct, and on win32-like machines, our output
+		 buffer is opened in textmode and will re-canonicalize
+		 line endings back to CRLF.  Since this code is just
+		 for handling armored keys, we don't have to worry
+		 about odd cases like CRCRCR and the like. */
+
+	      if(buf[i]!='\r')
+		fputc(buf[i],ctx->stream);
+	    }
 	}
     }
 
   return size*nmemb;
 }
-#endif
+
+void
+curl_writer_finalize(struct curl_writer_ctx *ctx)
+{
+  if(ctx->flags.armor)
+    {
+      if(ctx->armor_remaining==2)
+	{
+	  /* Top 6 bytes of ctx->armorctx[0] */
+	  fputc(B64[(ctx->armor_ctx[0]>>2)&0x3F],ctx->stream);
+	  /* Bottom 2 bytes of ctx->armor_ctx[0] and top 4 bytes of
+	     ctx->armor_ctx[1] */
+	  fputc(B64[(((ctx->armor_ctx[0]<<4)&0x30)
+		     |((ctx->armor_ctx[1]>>4)&0x0F))&0x3F],ctx->stream);
+	  /* Bottom 4 bytes of ctx->armor_ctx[1] */
+	  fputc(B64[((ctx->armor_ctx[1]<<2)&0x3C)],ctx->stream);
+	  /* Pad */
+	  fputc('=',ctx->stream);
+	}
+      else if(ctx->armor_remaining==1)
+	{
+	  /* Top 6 bytes of ctx->armor_ctx[0] */
+	  fputc(B64[(ctx->armor_ctx[0]>>2)&0x3F],ctx->stream);
+	  /* Bottom 2 bytes of ctx->armor_ctx[0] */
+	  fputc(B64[((ctx->armor_ctx[0]<<4)&0x30)],ctx->stream);
+	  /* Pad */
+	  fputc('=',ctx->stream);
+	  /* Pad */
+	  fputc('=',ctx->stream);
+	}
+
+      fprintf(ctx->stream,"\n"END);
+      ctx->flags.done=1;
+    }
+}
