@@ -15,7 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  */
 
 #include <config.h>
@@ -64,7 +65,7 @@ mpi_mulpowm( MPI res, MPI *basearray, MPI *exparray, MPI m)
     assert(t);
     assert( k < 10 );
 
-    G = m_alloc_clear( (1<<k) * sizeof *G );
+    G = xmalloc_clear( (1<<k) * sizeof *G );
     /* and calculate */
     tmp =  mpi_alloc( mpi_get_nlimbs(m)+1 );
     mpi_set_ui( res, 1 );
@@ -95,5 +96,5 @@ mpi_mulpowm( MPI res, MPI *basearray, MPI *exparray, MPI m)
     mpi_free(tmp);
     for(i=0; i < (1<<k); i++ )
 	mpi_free(G[i]);
-    m_free(G);
+    xfree(G);
 }
