@@ -255,7 +255,7 @@ print_utf8_string2 ( FILE *fp, const byte *p, size_t n, int delim )
 	buf = utf8_to_native ( p, n, delim );
 	/*(utf8 conversion already does the control character quoting)*/
 	fputs( buf, fp );
-	m_free( buf );
+	xfree( buf );
     }
     else
 	print_string( fp, p, n, delim );
@@ -269,7 +269,7 @@ print_utf8_string( FILE *fp, const byte *p, size_t n )
 
 /****************
  * This function returns a string which is suitable for printing
- * Caller must release it with m_free()
+ * Caller must release it with xfree()
  */
 char *
 make_printable_string( const byte *p, size_t n, int delim )
@@ -294,7 +294,7 @@ make_printable_string( const byte *p, size_t n, int delim )
     p = save_p;
     n = save_n;
     /* and now make the string */
-    d = buffer = m_alloc( buflen );
+    d = buffer = xmalloc( buflen );
     for( ; n; n--, p++ ) {
 	if( *p < 0x20 || (*p >= 0x7f && *p < 0xa0) || *p == delim ||
 	    (delim && *p=='\\')) {
@@ -313,7 +313,7 @@ make_printable_string( const byte *p, size_t n, int delim )
 		*d++ = '0';
 	    else {
 		sprintf(d, "x%02x", *p );
-		d += 2;
+		d += 3;
 	    }
 	}
 	else
@@ -326,10 +326,10 @@ make_printable_string( const byte *p, size_t n, int delim )
 int
 answer_is_yes_no_default( const char *s, int def_answer )
 {
-    /* NOTE TO TRANSLATOR: See doc/TRANSLATE about this string. */
+    /* TRANSLATORS: See doc/TRANSLATE about this string. */
     const char *long_yes = _("yes");
     const char *short_yes = _("yY");
-    /* NOTE TO TRANSLATOR: See doc/TRANSLATE about this string. */
+    /* TRANSLATORS: See doc/TRANSLATE about this string. */
     const char *long_no = _("no");
     const char *short_no = _("nN");
 
@@ -363,11 +363,11 @@ answer_is_yes( const char *s )
 int
 answer_is_yes_no_quit( const char *s )
 {
-    /* NOTE TO TRANSLATOR: See doc/TRANSLATE about this string. */
+    /* TRANSLATORS: See doc/TRANSLATE about this string. */
     const char *long_yes = _("yes");
-    /* NOTE TO TRANSLATOR: See doc/TRANSLATE about this string. */
+    /* TRANSLATORS: See doc/TRANSLATE about this string. */
     const char *long_no = _("no");
-    /* NOTE TO TRANSLATOR: See doc/TRANSLATE about this string. */
+    /* TRANSLATORS: See doc/TRANSLATE about this string. */
     const char *long_quit = _("quit");
     const char *short_yes = _("yY");
     const char *short_no = _("nN");
@@ -403,9 +403,9 @@ answer_is_yes_no_quit( const char *s )
 int
 answer_is_okay_cancel (const char *s, int def_answer)
 {
-  /* NOTE TO TRANSLATOR: See doc/TRANSLATE about this string. */
+  /* TRANSLATORS: See doc/TRANSLATE about this string. */
   const char *long_okay = _("okay|okay");
-  /* NOTE TO TRANSLATOR: See doc/TRANSLATE about this string. */
+  /* TRANSLATORS: See doc/TRANSLATE about this string. */
   const char *long_cancel = _("cancel|cancel");
   const char *short_okay = _("oO");
   const char *short_cancel = _("cC");
