@@ -20,7 +20,6 @@
  */
 
 #include <config.h>
-#ifdef USE_DNS_SRV
 #include <sys/types.h>
 #ifdef _WIN32
 #include <windows.h>
@@ -43,7 +42,8 @@
 #define T_SRV 33
 #endif
 
-static int priosort(const void *a,const void *b)
+static int
+priosort(const void *a,const void *b)
 {
   const struct srventry *sa=a,*sb=b;
   if(sa->priority>sb->priority)
@@ -65,7 +65,7 @@ getsrv(const char *name,struct srventry **list)
   *list=NULL;
 
   r=res_query(name,C_IN,T_SRV,answer,PACKETSZ);
-  if(r<sizeof(HEADER))
+  if(r<sizeof(HEADER) || r>PACKETSZ)
     return -1;
 
   if((((HEADER *)answer)->rcode)==NOERROR &&
@@ -89,7 +89,7 @@ getsrv(const char *name,struct srventry **list)
 	  struct srventry *srv=NULL;
 	  u16 type,class;
 
-	  *list=m_realloc(*list,(srvcount+1)*sizeof(struct srventry));
+	  *list=xrealloc(*list,(srvcount+1)*sizeof(struct srventry));
 	  memset(&(*list)[srvcount],0,sizeof(struct srventry));
 	  srv=&(*list)[srvcount];
 	  srvcount++;
@@ -216,18 +216,16 @@ getsrv(const char *name,struct srventry **list)
   return srvcount;
 
  noanswer:
-  m_free(*list);
+  xfree(*list);
   *list=NULL;
   return 0;
 
  fail:
-  m_free(*list);
+  xfree(*list);
   *list=NULL;
   return -1;
 }
 
-#endif /* USE_DNS_SRV */
-
 #ifdef TEST
 int
 main(int argc,char *argv[])
@@ -246,8 +244,14 @@ main(int argc,char *argv[])
       printf("\n");
     }
 
-  m_free(srv);
+  xfree(srv);
 
   return 0;
 }
 #endif /* TEST */
+
+/*
+Local Variables:
+compile-command: "cc -DTEST -I.. -I../include -Wall -g -o srv srv.c -lresolv libutil.a"
+End:
+*/
